import 'dart:convert';
import 'package:archive/archive.dart';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:lokalise_flutter_sdk/ota/api/downloader.dart';
import 'package:lokalise_flutter_sdk/ota/exceptions/api_exception.dart';
import 'package:lokalise_flutter_sdk/ota/exceptions/bundle_not_found_exception.dart';
import 'package:lokalise_flutter_sdk/ota/exceptions/update_exception.dart';
import 'package:lokalise_flutter_sdk/ota/model/bundle_archive.dart';
import 'package:lokalise_flutter_sdk/ota/model/bundle_info.dart';
import 'package:lokalise_flutter_sdk/ota/service/functions.dart';
import 'package:lokalise_flutter_sdk/ota/service/sdk_store.dart';

class Api {
  static const String domain = 'https://ota.lokalise.com';
  static Uri? uri;

  Api._();

  /// Request to Lokalise server for bundle info and data
  static Future<BundleArchive> getBundleData(String sdkToken, String projectId,
      bool preRelease, int trVersion, int appVersion) async {
    final preReleaseString = preRelease ? '&prerelease=true' : '';
    String url =
        '$domain/v3/lokalise/projects/$projectId/frameworks/flutter_sdk?appVersion=$appVersion&transVersion=$trVersion$preReleaseString';
    uri = Uri.parse(url);
    Map<String, String> headers = {
      'x-ota-api-token': sdkToken,
      'content-type': 'application/json'
    };
    final Client client = Client();
    BundleInfo resultBundleInfo = await bundleInfo(client, uri, headers);
    int transVersion = resultBundleInfo.transVersion;
    Archive archiveData = Archive();
    bool newRelease = false;
    if (transVersion != trVersion) {
      final bundleFile =
          await Downloader.downloadBundle(resultBundleInfo.bundleUrl);
      archiveData = ZipDecoder().decodeBytes(bundleFile);
      newRelease = true;
    }
    return BundleArchive(transVersion, archiveData, newRelease);
  }

  // Request to Lokalise server for bundle info
  static Future<BundleInfo> bundleInfo(
      http.Client client, uri, Map<String, String> headers) async {
    final response = await client.get(uri, headers: headers);

    if (response.statusCode == 200) {
      return BundleInfo.fromJson(jsonDecode(response.body)["data"]);
    } else if (response.statusCode == 204) {
      throw UpdateException();
    } else if (response.statusCode == 404) {
      SdkStore.removeStoredReleaseData();
      throw BundleNotFoundException();
    }

    throw ApiException('Failed to fetch bundle info', response.statusCode,
        Functions.formatJsonMessage(response.body));
  }
}
