import 'package:lokalise_flutter_sdk/ota/model/release_data.dart';
import 'package:lokalise_flutter_sdk/ota/model/stored_release_data.dart';
import 'package:lokalise_flutter_sdk/ota/platforms/platform.dart';
import 'package:lokalise_flutter_sdk/ota/service/logger.dart';

class SdkStore {
  static final _log = getLogger();
  static final _platform = Platform();

  SdkStore._();

  /// Save data to storage for all platforms
  static Future<void> releaseData(
      String projectId, ReleaseData releaseData) async {
    try {
      var storedReleaseData = StoredReleaseData(projectId, releaseData);
      await _platform.saveStoredReleaseData(storedReleaseData);
    } catch (e) {
      _log.w('Failed to cache translations.', e);
    }
  }

  /// Remove data from storage
  static Future<void> removeStoredReleaseData() async {
    try {
      await _platform.removeStoredReleaseData();
    } catch (e) {
      _log.w('Failed to clear cached translations.', e);
    }
  }

  /// Get data from storage by Project ID
  static Future<ReleaseData?> getReleaseData(String projectId) async {
    ReleaseData? releaseData;
    try {
      var storedData = await _platform.getStoredReleaseData();
      if (storedData == null) return null;
      if (storedData.projectId != projectId) {
        await _platform.removeStoredReleaseData();
        return null;
      }
      releaseData = storedData.releaseData;
    } catch (e) {
      _log.w('Failed to load cached translations.', e);
    }
    return releaseData;
  }
}
