import 'dart:io' as io;
import 'package:archive/archive_io.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:flutter_test/flutter_test.dart';
import 'package:lokalise_flutter_sdk/ota/service/bundle_data.dart';
import 'package:lokalise_flutter_sdk/ota/service/bundle_service.dart';
import 'package:mockito/annotations.dart';

@GenerateMocks([http.Client])

/// Bundle archive tests
void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  group('Bundle downloader test', () {
    test('returns an BundleData if extracted', () async {
      Archive bundleArchive = Archive();
      var encoder = ZipFileEncoder();
      encoder.create("test/testPath/test.zip");
      encoder.addDirectory(io.Directory("test/assets/locale"));
      encoder.close();
      final zipFile = io.File("test/testPath/test.zip");
      final bytes = await zipFile.readAsBytes();
      bundleArchive = ZipDecoder().decodeBytes(bytes);
      expect(await BundleService.bundleExtract(bundleArchive),
          isInstanceOf<BundleData>());
    });
    test('BundleArchive is empty', () async {
      Archive bundleArchive = Archive();
      var encoder = ZipFileEncoder();
      encoder.create("test/testPath/test.zip");
      encoder.close();
      final zipFile = io.File("test/testPath/test.zip");
      final bytes = await zipFile.readAsBytes();
      bundleArchive = ZipDecoder().decodeBytes(bytes);

      final result = await BundleService.bundleExtract(bundleArchive);
      expect(result.data.isEmpty, true);
    });
  });
}
